@@ -49,6 +49,7 @@ TRANS_MODE = trans_mode.from_waxs
 # this is the minimum intensity to be used for trans calculations
 WAXS_THRESH = 10
 
+
 # this is the scaling factor for indivudual curves that belong to the same sample
 # they are offset for clarity in the plots
 VOFFSET = 1.5
@@ -69,13 +70,16 @@ def get_font_size(size_index):
     return i-3,font_size_list[i]
         
 class Data1d:
-    def __init__(self, trandMode=None):
+    def __init__(self, transMode=None):
         self.comments = ""
         self.label = "data"
         self.overlaps = []
         self.raw_data = {}
         self.timestamp = None
-        self.trans = 0
+        self.trans_w = -1
+        self.trans_e = -1
+        self.trans = -1
+        self.transMode = transMode
         
     def load_from_2D(self, image, exp_para, qgrid, pre_process=None, 
                      mask=None, save_ave=False, debug=False, label=None):
@@ -123,18 +127,24 @@ class Data1d:
             self.save(image + ".ave", debug=debug)     
         
 
-    def set_trans(self, trans=-1, ref_trans=-1, transMode=None, 
-                  q_start=1.85, q_end=2.15, debug=False):
+    def set_trans(self, transMode=None, trans=-1, ref_trans=-1,
+                  calc_water_peak=False, q_start=1.85, q_end=2.15, debug=False):
         """
         normalize intensity, from trans to ref_trans
-        trans can be either from the beam center or water scattering
+        trans can be either from the beam center/beam stop or water scattering
+        sometimes the data may require both measures of transmitted intensity
+        
         this operation should be performed after SAXS/WAXS merge, because
-        1. SAXS and WAXS should have the same trans
-        2. if trans_mode is TRNAS_FROM_WAXS, the trans value needs to be calculated from WAXS data
+          1. SAXS and WAXS should have the same trans
+          2. if trans_mode is TRNAS_FROM_WAXS, the trans value needs to be calculated from WAXS data
+        
         """
-        if transMode is not None:
+        if transMode is None and self.transMode is not None:
+            transMode = self.transMode
+        else:
+            assert(isinstance(transMode, trans_mode))
             self.transMode = transMode
-        if self.transMode == trans_mode.from_waxs:
+        if self.transMode==trans_mode.from_waxs or calc_water_peak:
             # get trans for the near the maximum in the WAXS data
             # for solution scattering, hopefully this reflect the intensity of water scattering
             idx = (self.qgrid > q_start) & (self.qgrid < q_end)  # & (self.data>0.5*np.max(self.data))
@@ -149,35 +159,43 @@ class Data1d:
             if (self.data[idx]<WAXS_THRESH).all() and debug!='quiet':
                 print("the data points for trans calculation are below WAXS_THRESH: ", 
                       np.max(self.data[idx]), WAXS_THRESH)                
-            self.trans = np.sum(self.data[idx])
+            self.trans_w = np.sum(self.data[idx])
+            if self.transMode==trans_mode.from_waxs:
+                self.trans = self.trans_w 
             qavg = np.average(self.qgrid[idx])
-            if self.trans<1.0:
-                print('caluclated trans is %f, setting it artifically to WAXS_THRESH.' % self.trans)
-                self.trans = WAXS_THRESH
+            if self.trans_w<1.0:
+                print(f'caluclated trans is {self.trans_w}, setting it artifically to WAXS_THRESH.')
+                self.trans_w = WAXS_THRESH
             if debug==True:
                 print("using data near the high q end (q~%f)" % qavg, end=' ')
             self.comments += "# transmitted beam intensity from WAXS (q~%.2f)" % qavg
-        elif self.transMode == trans_mode.external:
-            if trans > 0:
-                self.comments += "# transmitted beam intensity is defined externally: %f"%trans
-                self.trans = trans
-            elif self.trans<0:
-                print("trans_mode is TRANS_EXTERNAL but a valid trans value is not provided")
-                raise Exception()
-        else:
-            raise Exception("invalid transmode: ", self.tMode)
+        if self.transMode==trans_mode.external and trans>=0: 
+            # if the trans value is specified, by definition transMode should be external
+            # if trans=-1, it is not meant to be the value to be set
+            #if trans<0:
+            #    print(f"Warning: {trans} is not a valid value for transmitted intensity.")
+            #    trans = 0
+            self.comments += f"# transmitted beam intensity given externally: {trans}"
+            self.trans_e = trans
+            self.trans = trans
+            #if self.transMode==trans_mode.external:
+            #    self.trans = trans
 
         self.comments += ": %f \n" % self.trans
         if debug==True:
             print("trans for %s set to %f" % (self.label, self.trans))
 
         if ref_trans > 0:
+            if self.trans<0:
+                print(f"cannot normalize intensity since data1d does not have a valid trans value: {self.trans}")
             self.comments += "# scattering intensity normalized to ref_trans = %f \n" % ref_trans
-            self.data *= ref_trans / self.trans
-            self.err *= ref_trans / self.trans
+            self.data *= ref_trans/self.trans
+            self.err *= ref_trans/self.trans
             for ov in self.overlaps:
-                ov['raw_data1'] *= ref_trans / self.trans
-                ov['raw_data2'] *= ref_trans / self.trans
+                ov['raw_data1'] *= ref_trans/self.trans
+                ov['raw_data2'] *= ref_trans/self.trans
+            self.trans_w *= ref_trans/self.trans
+            self.trans_e *= ref_trans/self.trans
             self.trans = ref_trans
             if debug==True:
                 print("normalized to %f" % ref_trans)
@@ -246,6 +264,8 @@ class Data1d:
                 raise Exception("\n1D sets cannot be averaged: qgrid mismatch")
 
             d0.trans += d1.trans
+            d0.trans_w += d1.trans_w
+            d0.trans_e += d1.trans_e
             if weighted:
                 er2 = (d1.err/wt[i])**2
                 d0.err += 1/er2
@@ -270,6 +290,8 @@ class Data1d:
             d0.label = common_name(d0.label, d1.label)
 
         d0.trans /= n
+        d0.trans_w /= n
+        d0.trans_e /= n
         if weighted:
             d0.data /= d0.err
             d0.err = 1./np.sqrt(d0.err)
@@ -384,6 +406,10 @@ class Data1d:
         self.data *= sc
         self.err *= sc
         self.trans *= sc
+        if self.trans_w>0:
+            self.trans_w *= sc
+        if self.trans_e>0:
+            self.trans_e *= sc
         self.comments += "# data is scaled by %f.\n" % sc
         if len(self.overlaps) != 0:
             for ov in self.overlaps:
@@ -481,7 +507,8 @@ class Data1d:
         self.comments += " scaled by %f\n" % sc
         self.comments += d1.comments.replace("# ", "## ")
 
-    def plot_Guinier(self, qs=0, qe=10, rg=15, fix_qe=False, ax=None, no_plot=False, fontsize="large"):
+    def plot_Guinier(self, qs=0, qe=10, rg=15, fix_qe=False, scale_wabs=-1,
+                     ax=None, no_plot=False, fontsize="large"):
         """ do Gunier plot, estimate Rg automatically
         qs specify the lower end of the q-range to perform the fit in
         rg is the optinal initial estimate
@@ -491,12 +518,17 @@ class Data1d:
         i_fs = get_font_size(fontsize)[0]
         # print self.data
 
+        scale = 1.0
+        if scale_wabs>0:
+            assert(self.trans_w>0)
+            scale *= scale_wabs/self.trans_w
+
         if no_plot==False:
             if ax is None:
                 ax = plt.gca()
             ax.set_xscale('linear')
             ax.set_yscale('log')
-            ax.errorbar(self.qgrid[idx] ** 2, self.data[idx], self.err[idx])
+            ax.errorbar(self.qgrid[idx]**2, self.data[idx]*scale, self.err[idx]*scale)
 
         cnt = 0
         t = self.qgrid[self.data > 0][0]
@@ -519,10 +551,14 @@ class Data1d:
             cnt += 1
             # print i0, rg
         td[1, :] = i0 * np.exp(-td[0, :]*rg*rg/3.)
-
+        n1 = len(self.qgrid[self.qgrid<qs])
+        n2 = len(self.qgrid)-len(self.qgrid[self.qgrid>qe])
+        fit_range = [n1,n2]
+        i0*=scale
+        
         if no_plot==False and rg>0.1:
             #ax.tick_params(axis='y', labelleft=False)    
-            ax.plot([td[0, 0], td[0, -1]], [td[1, 0], td[1, -1]], "ro")
+            ax.plot([td[0, 0], td[0, -1]], [td[1, 0]*scale, td[1, -1]*scale], "ro")
             ax.plot(self.qgrid**2, i0*np.exp(-(self.qgrid*rg)**2/3))
             ax.set_ylabel("$I$", fontsize=get_font_size(i_fs)[1])
             ax.set_xlabel("$q^2 (\AA^{-2})$", fontsize=get_font_size(i_fs)[1])
@@ -534,7 +570,7 @@ class Data1d:
             if i0>0:
                 ax.set_ylim(top=i0*2, bottom=i0*np.exp(-(qe*rg)**2/3)/2)
         # print "I0=%f, Rg=%f" % (i0,rg)
-        return (i0, rg)
+        return (i0, rg, fit_range) # include fit range to be more compatible with ATSAS
 
     def plot_pr(self, i0, rg, qmax=5., dmax=200., ax=None, fontsize='large'):
         """ calculate p(r) function
@@ -571,31 +607,42 @@ class Data1d:
         ax.set_ylabel("$P(r)$", fontsize=get_font_size(i_fs)[1])
         # plt.subplots_adjust(bottom=0.15)
 
-    def save(self, fn, nz=True, save_comments=False, debug=False, footer=None):
+    def save(self, fn, nz=True, scale_wabs=-1,
+             save_comments=False, debug=False, footer=None):
         """
         should save all the relevant information, such as scaling, merging, averaging
         save data points with non-zero intensity only if nz==1
         """
-        qidi = np.vstack((self.qgrid, self.data, self.err))
+        if scale_wabs>0:
+            assert(self.trans_w>0)
+            scale_wabs /= self.trans_w
+            qidi = np.vstack((self.qgrid, self.data*scale_wabs, self.err*scale_wabs))
+        else:
+            qidi = np.vstack((self.qgrid, self.data, self.err))
         if nz:
             qidi = qidi[:, self.data != 0]
         if debug==True:
             print("saving file: %s, nz=%d" % (fn, nz))
-        np.savetxt(fn, qidi.T, "%12.5f")
+        np.savetxt(fn, qidi.T, "%8.4f   %8.3e   %8.3e")
         if save_comments or footer is not None:
             ff = open(fn, "a")
             if save_comments:
                 ff.write(self.comments)
+                if scale_wabs>0:
+                    ff.write("# converted to abs scale by applying a scaling factor of {scale_wabs:.2e}\n")
             elif footer is not None:
                 ff.write(footer)
             ff.close()
 
-    def plot(self, ax=None, scale=1., fontsize='large'):
+    def plot(self, ax=None, scale=1., fontsize='large', scale_wabs=-1):
         i_fs = get_font_size(fontsize)[0]
         if ax is None:
             plt.figure()
             plt.subplots_adjust(bottom=0.15)
             ax = plt.gca()
+        if scale_wabs>0:
+            assert(self.trans_w>0)
+            scale *= scale_wabs/self.trans_w
         ax.set_xlabel("$q (\AA^{-1})$", fontsize=get_font_size(i_fs)[1])
         ax.set_ylabel("$I$", fontsize=get_font_size(i_fs)[1])
         ax.set_xscale('log')
@@ -616,7 +663,7 @@ def calculate(ds0, ds1):
     diff_coef = np.sum(np.abs(np.subtract(ds0, ds1)))  # How different the datasets are
     return diff_coef            
             
-def filter_by_similarity(datasets, similarity_threshold=0.5):
+def filter_by_similarity(datasets, similarity_threshold=0.5, debug=False):
     
     if len(datasets)==1:
         return datasets,None
@@ -646,7 +693,8 @@ def filter_by_similarity(datasets, similarity_threshold=0.5):
 
     # No valid candidate, return all the data
     if np.array_equal(number_of_simil_per_column, np.ones(number_of_datasets)):
-        print("No dataset with similarity level below threshold. Returning everything.")
+        if debug is True:
+            print("No dataset with similarity level below threshold. Returning everything.")
         return datasets, []
 
     best_datasets_column = np.argmax(number_of_simil_per_column)
@@ -726,7 +774,7 @@ def merge_detectors(fns, detectors, qgrid, reft=-1, plot_data=False, save_ave=Fa
         s0.data[idx] /= c_tot[idx]
         s0.err[idx] /= c_tot[idx]
         s0.transMode = transMode
-        s0.set_trans(trans=trans, ref_trans=reft, transMode=transMode, debug=debug)
+        s0.set_trans(transMode, trans=trans, ref_trans=reft, debug=debug)
         s0.label = label
         s0.comments = comments # .replace("# ", "## ")
 
@@ -760,7 +808,7 @@ def average(fns, detectors, qgrid, reft=-1, plot_data=False, save1d=0, ax=None,
                          ax, qmax, qmin, fix_scale, debug=debug)
     t1 = time.time()
     if filter_datasets:
-        ss, invalids = filter_by_similarity(ss, similarity_threshold=similarity_threshold)
+        ss, invalids = filter_by_similarity(ss, similarity_threshold=similarity_threshold, debug=debug)
         # TODO: Insert warning/exception when the number of datasets discarded is high.
         # TODO: Define the % of discarded to result in a error.
         if debug!='quiet':
@@ -828,7 +876,7 @@ def process(sfns, bfns, detectors, qgrid, qmax=-1, qmin=-1,
 def analyze(d1, qstart, qend, fix_qe, qcutoff, dmax):
     plt.figure(figsize=(14, 5.5))
     plt.subplot(121)
-    (I0, Rg) = d1.plot_Guinier(qs=qstart, qe=qend, fix_qe=fix_qe)
+    I0,Rg,_ = d1.plot_Guinier(qs=qstart, qe=qend, fix_qe=fix_qe)
 
     print("I0=%f, Rg=%f" % (I0, Rg))
 
@@ -857,8 +905,11 @@ def estimate_scaling_factor(d1s, d1b,
     idx = (d1s.qgrid>q_min) & (d1s.qgrid<q_max) & (d1s.data>0) & (d1b.data>0)
     md1s = d1s.data[idx]
     md1b = d1b.data[idx]
-    md1sm = smooth(md1s, half_window_len=smoothing_width)
-    md1bm = smooth(md1b, half_window_len=smoothing_width)
+    try:
+        md1sm = smooth(md1s, half_window_len=smoothing_width)
+        md1bm = smooth(md1b, half_window_len=smoothing_width)
+    except:
+        return 1.0      # problems like incomplete data can cause smooth() to fail, no point to continue
     mq = d1s.qgrid[idx]
     sc = 0.9
     prec0 = 2
@@ -881,12 +932,16 @@ def estimate_scaling_factor(d1s, d1b,
             #elif std1<std0:
             #    std0 = std1
             if sp1>sp0+s_thresh:
+                if debug:
+                    print(f"span exceeded threshold: {sp1}, {sp0}+{s_thresh}")
                 break
             # assume that data*q should have a lower bound
             # this should work better on the smoothed data
             td1 = (md1sm-md1bm*sc1)*mq
             q_Imin = mq[td1.argmin()]
             if q_Imin>1.7:  # under the water peak, indication of over-subtraction
+                if debug:
+                    print(f"qxI min under water peak: q={q_Imin:.3f}")
                 break
             sc = sc1
             if debug:
